@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2021 A S Lewis
+# Copyright (C) 2019-2022 A S Lewis
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -281,7 +281,7 @@ video_option_setup_list = [
     '400',      'mp4 [1440p] <400>',                        False,
     '401',      'mp4 [2160p] <401>',                        False,
     '402',      'mp4 [2880p] <402>',                        False,
-    '571',      'mp4 [8k] <571',                            False,
+    '571',      'mp4 [8k] <571>',                           False,
     '43',       'webm [360p] <43>',                         False,
     '44',       'webm [480p] <44>',                         False,
     '45',       'webm [720p] <45>',                         False,
@@ -417,10 +417,13 @@ FILE_SIZE_UNIT_LIST = [
 ]
 
 language_setup_list = [
-    # ISO 639-1 Language Codes
+    # ISO 639-1 Language Codes (with one extra key-value pair to handle live
+    #   chat)
     # English is top of the list, because it's the default setting in
     #   options.OptionsManager
-    'English', 'en',
+    # NB These values must not contain square brackets [...]
+    _('English'), 'en',
+    'YouTube live chat', 'live_chat',
     'Abkhazian', 'ab',
     'Afar', 'aa',
     'Afrikaans', 'af',
@@ -461,7 +464,7 @@ language_setup_list = [
     'Czech', 'cs',
     'Danish', 'da',
     'Divehi, Dhivehi, Maldivian', 'dv',
-    'Dutch', 'nl',
+    _('Dutch'), 'nl',
     'Dzongkha', 'dz',
     'Esperanto', 'eo',
     'Estonian', 'et',
@@ -511,7 +514,7 @@ language_setup_list = [
     'Kyrgyz', 'ky',
     'Komi', 'kv',
     'Kongo', 'kg',
-    'Korean', 'ko',
+    _('Korean'), 'ko',
     'Kurdish', 'ku',
     'Kwanyama', 'kj',
     'Lao', 'lo',
@@ -626,6 +629,7 @@ DIALOGUE_ICON_DICT = {
     'ready_icon': 'ready_icon_64.png',
     'setup_classic_icon': 'setup_classic_icon.png',
     'system_icon': 'system_icon_64.png',
+    'yt_icon': 'yt_icon_32.png',
     'yt_remind_icon_en': 'yt_remind_icon_en.png',
     'yt_remind_icon_kr': 'yt_remind_icon_kr.png',
     'yt_remind_icon_nl': 'yt_remind_icon_nl.png',
@@ -703,55 +707,37 @@ TOOLBAR_ICON_DICT = {
 }
 
 LARGE_ICON_DICT = {
-    'video_both_large': 'video_both.png',
-    'video_left_large': 'video_left.png',
-    'video_none_large': 'video_none.png',
-    'video_right_large': 'video_right.png',
-
-    'channel_both_large': 'channel_both.png',
-    'channel_left_large': 'channel_left.png',
-    'channel_none_large': 'channel_none.png',
-    'channel_right_large': 'channel_right.png',
-
-    'playlist_both_large': 'playlist_both.png',
-    'playlist_left_large': 'playlist_left.png',
-    'playlist_none_large': 'playlist_none.png',
-    'playlist_right_large': 'playlist_right.png',
-
-    'folder_both_large': 'folder_yellow_both.png',
-    'folder_left_large': 'folder_yellow_left.png',
-    'folder_none_large': 'folder_yellow_none.png',
-    'folder_right_large': 'folder_yellow_right.png',
-
-    'folder_private_both_large': 'folder_red_both.png',
-    'folder_private_left_large': 'folder_red_left.png',
-    'folder_private_none_large': 'folder_red_none.png',
-    'folder_private_right_large': 'folder_red_right.png',
-
-    'folder_fixed_both_large': 'folder_green_both.png',
-    'folder_fixed_left_large': 'folder_green_left.png',
-    'folder_fixed_none_large': 'folder_green_none.png',
-    'folder_fixed_right_large': 'folder_green_right.png',
-
-    'folder_temp_both_large': 'folder_blue_both.png',
-    'folder_temp_left_large': 'folder_blue_left.png',
-    'folder_temp_none_large': 'folder_blue_none.png',
-    'folder_temp_right_large': 'folder_blue_right.png',
-
-    'folder_no_parent_both_large': 'folder_black_both.png',
-    'folder_no_parent_left_large': 'folder_black_left.png',
-    'folder_no_parent_none_large': 'folder_black_none.png',
-    'folder_no_parent_right_large': 'folder_black_right.png',
-
+    'attention_large': 'attention.png',
+    'channel_large': 'channel.png',
     'copy_large': 'copy.png',
+    'cursor_large': 'cursor.png',
+    'error_large': 'error.png',
+    'folder_large': 'folder_yellow.png',
+    'folder_fixed_large': 'folder_green.png',
+    'folder_no_parent_large': 'folder_black.png',
+    'folder_private_large': 'folder_red.png',
+    'folder_temp_large': 'folder_blue.png',
     'hand_left_large': 'hand_left.png',
     'hand_right_large': 'hand_right.png',
     'limits_off_large': 'limits_off.png',
     'limits_on_large': 'limits_on.png',
+    'playlist_large': 'playlist.png',
     'question_large': 'question.png',
+    'video_large': 'video.png',
     'warning_large': 'warning.png',
 }
 
+LARGE_ICON_COMPOSITE_LIST = [
+    'channel_large',
+    'folder_large',
+    'folder_fixed_large',
+    'folder_no_parent_large',
+    'folder_private_large',
+    'folder_temp_large',
+    'playlist_large',
+    'video_large',
+]
+
 SMALL_ICON_DICT = {
     'video_small': 'video.png',
     'channel_small': 'channel.png',
@@ -761,6 +747,7 @@ SMALL_ICON_DICT = {
     'archived_small': 'archived.png',
     'arrow_up_small': 'arrow_up.png',
     'arrow_down_small': 'arrow_down.png',
+    'attention_small': 'attention.png',
     'check_small': 'check.png',
     'comment_small': 'comment.png',
     'debut_now_small': 'debut_now.png',
@@ -775,14 +762,18 @@ SMALL_ICON_DICT = {
     'folder_blue_small': 'folder_blue.png',
     'folder_green_small': 'folder_green.png',
     'folder_red_small': 'folder_red.png',
+    'keyboard_small': 'keyboard.png',
     'likes_small': 'likes.png',
     'have_file_small': 'have_file.png',
     'live_now_small': 'live_now.png',
+    'live_old_small': 'live_old.png',
+    'live_old_no_file_small': 'live_old_no_file.png',
     'live_wait_small': 'live_wait.png',
     'no_file_small': 'no_file.png',
     'slice_small': 'slice.png',
     'split_file_small': 'split_file.png',
     'stamp_small': 'stamp.png',
+    'subs_small': 'subs.png',
     'system_error_small': 'system_error.png',
     'system_warning_small': 'system_warning.png',
     'unavailable_small': 'unavailable.png',
@@ -820,10 +811,16 @@ THUMB_ICON_DICT = {
     'thumb_default_medium': 'thumb_default_medium.png',
     'thumb_default_large': 'thumb_default_large.png',
     'thumb_default_enormous': 'thumb_default_enormous.png',
+
+    'thumb_block_tiny': 'thumb_block_tiny.png',
+    'thumb_block_small': 'thumb_block_small.png',
+    'thumb_block_medium': 'thumb_block_medium.png',
+    'thumb_block_large': 'thumb_block_large.png',
+    'thumb_block_enormous': 'thumb_block_enormous.png',
 }
 
 EXTERNAL_ICON_DICT = {
-    'ytdl-gui': 'youtube-dl-gui.png',
+    'ytdl_gui': 'youtube-dl-gui.png',
 }
 
 # (Replaces system stock icons, if not available)
@@ -886,6 +883,17 @@ else:
         'system_icon_512.png',
     ]
 
+CONFIG_WIN_ICON_LIST = [
+    'config_icon_16.png',
+    'config_icon_24.png',
+    'config_icon_32.png',
+    'config_icon_48.png',
+    'config_icon_64.png',
+    'config_icon_128.png',
+    'config_icon_256.png',
+    'config_icon_512.png',
+]
+
 def do_translate(config_flag=False):
 
     """Function called for the first time below, setting various values.
