@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2021 A S Lewis
+# Copyright (C) 2019-2022 A S Lewis
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,6 +34,8 @@ import formats
 import mainapp
 import media
 import utils
+# Use same gettext translations
+from mainapp import _
 
 
 # Classes
@@ -369,6 +371,8 @@ class OptionsManager(object):
         trim_filenames (int): Limit the filename length (excluding extension)
             to the specified number of characters
 
+        no_overwrites (bool): If True, does not overwrite any files
+
         force_overwrites (bool): If True, overwrites all video and metadata
             files. This option includes '--no-continue' (for which there is no
             Tartube download option)
@@ -380,6 +384,16 @@ class OptionsManager(object):
         no_clean_info_json (bool): If True, writes all fields to the infojson
             (default is to remove some private fields)
 
+        no_cookies (bool): If True, does not read/dump cookies from/to file
+            (default)
+
+        cookies_from_browser (str): The name of the browser and (optionally)
+            the name/path of the profile to load cookies from; a string in the
+            from BROWSER[+KEYRING][:PROFILE]
+
+        no_cookies_from_browser (bool): If true, does not load cookies from the
+            browser (default)
+
         [Internet Shortcut Options]
 
         write_link (bool): If True, writes an internet shortcut file, depending
@@ -601,11 +615,15 @@ class OptionsManager(object):
         # Unique ID for this options manager
         self.uid = uid
         # A non-unique name for this options manager. Managers that are
-        #   attached to a media data object have the same name as that object.
-        #   (The name is not unique because, for example, videos could have the
+        #   attached to a media data object have the same name as that object
+        # (The name is not unique because, for example, videos could have the
         #   same name as a channel; it's up to the user to avoid duplicate
         #   names)
+        # Empty strings are not valid as names
         self.name = name
+        # A short description, intended for use in the Drag and Drop tab
+        # Empty strings are valid as descriptions
+        self.descrip = name
         # If this object is attached to a media data object, the .dbid of that
         #   object; otherwise None
         self.dbid = dbid
@@ -693,6 +711,11 @@ class OptionsManager(object):
         Resets (or initialises) self.options_dict to its default state.
         """
 
+        if os.name != 'nt':
+            windows_filenames_flag = True
+        else:
+            windows_filenames_flag = False
+
         self.options_dict = {
             # OPTIONS
             'ignore_errors': True,
@@ -737,7 +760,7 @@ class OptionsManager(object):
             'nomtime': False,
             'write_description': True,
             'write_info': True,
-            'write_annotations': True,
+            'write_annotations': False,
             'cookies_path': '',
             # THUMBNAIL IMAGES
             'write_thumbnail': True,
@@ -799,11 +822,15 @@ class OptionsManager(object):
             'concurrent_fragments': 1,
             'throttled_rate': 0,
             # (Filesystem Options)
-            'windows_filenames': False,
+            'windows_filenames': windows_filenames_flag,
             'trim_filenames': 0,
+            'no_overwrites': False,
             'force_overwrites': False,
             'write_playlist_metafiles': False,
             'no_clean_info_json': False,
+            'no_cookies': False,
+            'cookies_from_browser': '',
+            'no_cookies_from_browser': True,
             # (Internet Shortcut Options)
             'write_link': False,
             'write_url_link': False,
@@ -861,14 +888,35 @@ class OptionsManager(object):
         }
 
 
-    def set_classic_mode_options(self):
+    def set_general_options(self):
 
-        """Called by mainapp.TartubeApp.apply_classic_download_options().
+        """Called by mainapp.TartubeApp.start().
 
-        When the user applies download options in the Classic Mode tab, a few
-        options should have different default values; this function sets them.
+        Configures this object with a suitable description.
         """
 
+        self.descrip = _('General (default) download options')
+
+
+    def set_classic_mode_options(self, no_descrip_flag=False):
+
+        """Called by mainapp.TartubeApp.start() and
+        .apply_classic_download_options().
+
+        Also called by self.set_mp3_options().
+
+        Configures this object for use in the Classic Mode tab.
+
+        Args:
+
+            no_descrip_flag (bool): Set to True when called from
+                self.set_mp3_options()
+
+        """
+
+        if not no_descrip_flag:
+            self.descrip = _('Download options for the Classic Mode tab')
+
         self.options_dict['write_description'] = False
         self.options_dict['write_info'] = False
         self.options_dict['write_annotations'] = False
@@ -890,6 +938,24 @@ class OptionsManager(object):
         self.options_dict['sim_keep_thumbnail'] = False
 
 
+    def set_mp3_options(self):
+
+        """Called by mainapp.TartubeApp.start().
+
+        Configures this object to download MP3s.
+        """
+
+        self.descrip = _('Download and convert to MP3 (requires FFmpeg)')
+
+        # (Actual downloads take place in the Classic Mode tab, so use the same
+        #   file options)
+        self.set_classic_mode_options(True)
+
+        # (Convert everything to MP3)
+        self.options_dict['extract_audio'] = True
+        self.options_dict['audio_format'] = 'mp3'
+
+
     # Set accessors
 
 
@@ -940,7 +1006,7 @@ class OptionsParser(object):
             # -i, --ignore-errors
             OptionHolder('ignore_errors', '-i', False),
             # --abort-on-error
-            OptionHolder('abort_on_error', '--abort-on-error ', False),
+            OptionHolder('abort_on_error', '--abort-on-error', False),
             # NETWORK OPTIONS
             # --proxy URL
             OptionHolder('proxy', '--proxy', ''),
@@ -1043,7 +1109,7 @@ class OptionsParser(object):
                 False,
             ),
             # --prefer-insecure
-            OptionHolder('prefer_insecure', '--prefer-insecure ', False),
+            OptionHolder('prefer_insecure', '--prefer-insecure', False),
             # --user-agent UA
             OptionHolder('user_agent', '--user-agent', ''),
             # --referer URL
@@ -1162,6 +1228,8 @@ class OptionsParser(object):
             OptionHolder('windows_filenames', '--windows-filenames', False),
             # --trim-filenames LENGTH
             OptionHolder('trim_filenames', '--trim-filenames', 0),
+            # --no-overwrites
+            OptionHolder('no_overwrites', '--no-overwrites', False),
             # --force-overwrites
             OptionHolder('force_overwrites', '--force-overwrites', False),
             # --write-playlist-metafiles
@@ -1172,6 +1240,16 @@ class OptionsParser(object):
             ),
             # --no-clean-infojson
             OptionHolder('no_clean_info_json', '--no-clean-infojson', False),
+            # --no-cookies
+            OptionHolder('no_cookies', '--no-cookies', False),
+            # --cookies-from-browser BROWSER[+KEYRING][:PROFILE]
+            OptionHolder('cookies_from_browser', '--cookies-from-browser', ''),
+            # --no-cookies-from-browser
+            OptionHolder(
+                'no_cookies_from_browser',
+                '--no-cookies-from-browser',
+                True,
+            ),
             # (Internet Shortcut Options)
             # --write-link
             OptionHolder('write_link', '--write-link', False),
@@ -1696,23 +1774,43 @@ class OptionsParser(object):
                 or operation_type == 'classic_custom'
             ) and media_data_obj.dummy_format is not None:
 
+                # A string specifying the media format to download, or None if
+                #   the user didn't specify one
+                # The string is made up of three optional components in a fixed
+                #   order and separated by underlines: 'convert', the video/
+                #   audio format, and the video resolution, for example 'mp4',
+                #   'mp4_720p', 'convert_mp4_720p'
+                # Valid values are those specified by formats.VIDEO_FORMAT_LIST
+                #   formats.AUDIO_FORMAT_LIST and formats.VIDEO_RESOLUTION_LIST
                 format_str = media_data_obj.dummy_format
                 convert_flag = False
-                # If format_str is not None, then it is one of Tartube's
-                #   standard media formats, or one of those values preceded by
-                #   'convert_'. Remove the trailing text, if found
-                match = re.search('^convert_(.*)$', format_str)
-                if match:
-                    format_str = match.group(1)
+                this_format = None
+                this_res = None
+
+                split_list = format_str.split('_')
+                if split_list and split_list[0] == 'convert':
+                    split_list.pop(0)
                     convert_flag = True
 
-                if not convert_flag:
+                if split_list \
+                and (
+                    split_list[0] in formats.VIDEO_FORMAT_LIST \
+                    or split_list[0] in formats.AUDIO_FORMAT_LIST
+                ):
+                    this_format = split_list.pop(0)
 
-                    # Download the video in the specified format, if available
+                if split_list \
+                and split_list[0] in formats.VIDEO_RESOLUTION_LIST:
+                    this_res = formats.VIDEO_RESOLUTION_DICT[split_list.pop(0)]
+
+                if not convert_flag and this_res is None:
+
+                    # Download the video in the specified format and
+                    #   resolution, if available
 
                     # Ignore all video/audio formats except the one specified
                     #   by the user in the Classic Mode tab
-                    copy_dict['video_format'] = format_str
+                    copy_dict['video_format'] = this_format
                     copy_dict['all_formats'] = False
                     copy_dict['video_format_list'] = []
                     copy_dict['video_format_mode'] = ''
@@ -1721,29 +1819,32 @@ class OptionsParser(object):
                     # v2.1.009: Since the user doesn't have the possibility of
                     #   setting the -f and --merge-output-format options to the
                     #   same value (e.g. 'mp4'), we must do so artificially
-                    copy_dict['merge_output_format'] = format_str
+                    copy_dict['merge_output_format'] = this_format
 
                     return
 
-                elif format_str in formats.VIDEO_FORMAT_DICT:
+                elif this_format is not None \
+                and this_format in formats.VIDEO_FORMAT_DICT:
 
                     # Converting video formats requires post-processing
                     # Ignore all video/audio formats except the one specified
                     #   by the user in the Classic Mode tab
-                    copy_dict['video_format'] = '0'
+                    if this_res is None:
+                        copy_dict['video_format'] = '0'
+                    else:
+                        copy_dict['video_format'] \
+                        = 'bestvideo[height<=?' + this_res + ']' \
+                        + '+bestaudio/best[height<=?' + this_res + ']'
+
                     copy_dict['all_formats'] = False
                     copy_dict['video_format_list'] = []
                     copy_dict['video_format_mode'] = ''
-                    copy_dict['recode_video'] = ''
-
-                    # v2.1.009: Since the user doesn't have the possibility of
-                    #   setting the -f and --merge-output-format options to the
-                    #   same value (e.g. 'mp4'), we must do so artificially
-                    copy_dict['merge_output_format'] = format_str
+                    copy_dict['recode_video'] = this_format
 
                     return
 
-                elif format_str in formats.AUDIO_FORMAT_DICT:
+                elif this_format is not None \
+                and this_format in formats.AUDIO_FORMAT_DICT:
 
                     # Converting audio formats requires post-processing
                     copy_dict['video_format'] = '0'
@@ -1751,12 +1852,23 @@ class OptionsParser(object):
                     copy_dict['video_format_list'] = []
                     copy_dict['video_format_mode'] = ''
                     copy_dict['extract_audio'] = True
-                    copy_dict['audio_format'] = format_str
+                    copy_dict['audio_format'] = this_format
                     copy_dict['recode_video'] = ''
 
                     return
 
+                elif this_format is None and this_res is not None:
+
+                    copy_dict['video_format'] = \
+                    'bestvideo[height<=?' + this_res + ']' \
+                    + '+bestaudio/best[height<=?' + this_res + ']'
+                    copy_dict['all_formats'] = False
+                    copy_dict['video_format_list'] = []
+                    copy_dict['video_format_mode'] = ''
+                    copy_dict['recode_video'] = ''
+
             # Special case: for broadcasting livestreams, use only HLS
+            # !!! DEBUG
             # v2.0.067: Downloading livestreams doesn't work at all for me, so
             #   I'm not sure whether this is appropriate, or not. Once it's
             #   fixed, perhaps we can offer the user a choice of formats
